import fire
import os
from pywinschedule.watcher import Watcher
from pywinschedule.utils.admin import isUserAdmin,runAsAdmin
if not isUserAdmin():
    runAsAdmin()

class CLI:
    @classmethod
    def serve(cls,path):
        if not os.path.exists(path):
            os.makedirs(path)
        assert os.path.isdir(path)
        from win32com.shell import shell, shellcon
        def startupdirectory():
            return shell.SHGetFolderPath(
                0,
                shellcon.CSIDL_COMMON_STARTUP,
                0,  # null access token (no impersonation)
                0  # want current value, shellcon.SHGFP_TYPE_CURRENT isn't available, this seems to work
            )
        STARTUP_PATH=startupdirectory()
        bat_path=os.path.join(STARTUP_PATH,'.bat')
        print('Writing to %s'%(bat_path))
        with open(bat_path,'w') as f:
            f.write('pywinschedule watch %s'%(path))
        cls.watch(path)
    @classmethod
    def watch(cls,path):
        watcher=Watcher(path)
        watcher.start()


def main():
    fire.Fire(CLI)

if __name__ == '__main__':
    main()


import os

_root=os.path.dirname(__file__)
_data_dir=os.path.join(_root,'data')

NSSM_PATH= os.path.join(_data_dir,'nssm.exe')

class NSSM:
    def __init__(self,path):
        self.path=path
    def asAdmin(self):
        pass
        # code below might be useless
        # if not isUserAdmin():
        #     runAsAdmin()
    def updateService(self,name,executable,params=''):
        self.setServiceExecutable(name,executable)
        self.setServiceParams(name,params)
    def installService(self,name,executable,params=''):
        self.asAdmin()
        os.system('%s install %s "%s" "%s"'%(self.path,name,executable,params))
    def setServiceExecutable(self,name,executable):
        self.asAdmin()
        os.system('%s set %s Application "%s"' % (self.path, name, executable))
    def setServiceParams(self,name,params):
        self.asAdmin()
        os.system('%s set %s AppParameters "%s"' % (self.path, name, params))
    def setServiceWorkdir(self,name,work_dir):
        self.asAdmin()
        os.system('%s set %s AppDirectory "%s"' % (self.path, name, work_dir))
    def startService(self,name):
        self.asAdmin()
        os.system('%s start %s' % (self.path, name))
    def stopService(self,name):
        self.asAdmin()
        os.system('%s stop %s' % (self.path, name))
    def removeService(self,name):
        self.asAdmin()
        os.system('%s remove %s confirm' % (self.path, name))

nssm=NSSM(NSSM_PATH)
def demo():

    # nssm.installService('MyService', sys.executable,r"D:\Projects\PythonProjects\pywinschedule\demo\demo_service.py")
    nssm.startService('MyService')
    # nssm.stopService('MyService')


if __name__ == '__main__':
    demo()


import os
import dateutil.parser
import win32com.client
import datetime

from pywinschedule.config import USER_HOME,HOME


def generate_hash(s, times=1):
    assert times >= 1
    import hashlib
    m = hashlib.md5()

    def gen():
        m.update(s.encode('utf-8'))
        return m.hexdigest()[:10]

    for i in range(times):
        data = gen()
    return data
def add_executable_to_windows_task_scheduler(path,daytime,params='',trigger_type='once',name='some task'):
    trigger_type_dict={
        'once':1,
        'daily':2,
        'weekly':3,
        'monthly':4,
    }


    scheduler = win32com.client.Dispatch('Schedule.Service')
    scheduler.Connect()
    root_folder = scheduler.GetFolder('\\')
    task_def = scheduler.NewTask(0)

    # Create trigger
    # start_time = datetime.datetime.now() + datetime.timedelta(minutes=1)
    start_time = daytime if  isinstance(daytime,datetime.datetime) else dateutil.parser.parse(daytime)

    TASK_TRIGGER_TIME = trigger_type_dict[trigger_type]
    trigger = task_def.Triggers.Create(TASK_TRIGGER_TIME)
    trigger.StartBoundary = start_time.isoformat()

    # Create action
    TASK_ACTION_EXEC = 0
    action = task_def.Actions.Create(TASK_ACTION_EXEC)
    action.ID = name
    action.Path = path
    # action.Path = r'python.exe'
    action.Arguments = params

    # Set parameters
    task_def.RegistrationInfo.Description = name
    task_def.Settings.Enabled = True
    task_def.Settings.StopIfGoingOnBatteries = False


    # set condition
    task_def.Settings.DisallowStartIfOnBatteries=False

    # Register task
    # If task already exists, it will be updated
    TASK_CREATE_OR_UPDATE = 6
    TASK_LOGON_NONE = 0
    root_folder.RegisterTaskDefinition(
        name,  # Task name
        task_def,
        TASK_CREATE_OR_UPDATE,
        '',  # No user
        '',  # No password
        TASK_LOGON_NONE)


def schedule_python_script(path,work_dir=None,daytime=None,params='',trigger_type='once',name='some_task'):
    path=os.path.abspath(path)
    if not os.path.exists(HOME):
        os.makedirs(HOME)
    if not work_dir:
        work_dir=HOME
    if not daytime:
        daytime= datetime.datetime.now() + datetime.timedelta(minutes=1)
    bat_path=os.path.join(HOME,'schedule_script_%s.bat'%(generate_hash(name)))
    with open(bat_path,'w') as f:
        f.write('cd %s\npython.exe %s %s'%(work_dir,path,params))
    add_executable_to_windows_task_scheduler(bat_path,daytime,trigger_type=trigger_type,name=name)

def demo():
    schedule_python_script('./hi.py', name='打招呼')

if __name__ == '__main__':
    demo()


import socket

import win32serviceutil

import servicemanager
import win32event
import win32service
import shutil
import os
def _get_site_packages_path():
    import site
    paths=site.getsitepackages()
    for p in paths:
        if 'site-packages' in p:
            return p

def fix_pywin32_bug():
    "copy pywintypes310.dll from pywin32_system32 to win32"
    dll_name='pywintypes310.dll'
    site_packages_path=_get_site_packages_path()
    src_dll_path=os.path.join(site_packages_path,'pywin32_system32',dll_name)
    dst_dll_path=os.path.join(site_packages_path,'win32',dll_name)
    if not os.path.exists(dst_dll_path):
        shutil.copy(src_dll_path,dst_dll_path)

class WinService(win32serviceutil.ServiceFramework):
    '''Base class to create winservice in Python'''

    _svc_name_ = 'pythonService'
    _svc_display_name_ = 'Python Service'
    _svc_description_ = 'Python Service Description'

    @classmethod
    def parse_command_line(cls):
        '''
        ClassMethod to parse the command line
        '''
        fix_pywin32_bug()
        win32serviceutil.HandleCommandLine(cls)

    def __init__(self, args):
        '''
        Constructor of the winservice
        '''
        win32serviceutil.ServiceFramework.__init__(self, args)
        self.hWaitStop = win32event.CreateEvent(None, 0, 0, None)
        socket.setdefaulttimeout(60)

    def SvcStop(self):
        '''
        Called when the service is asked to stop
        '''
        self.stop()
        self.ReportServiceStatus(win32service.SERVICE_STOP_PENDING)
        win32event.SetEvent(self.hWaitStop)

    def SvcDoRun(self):
        '''
        Called when the service is asked to start
        '''
        self.start()
        servicemanager.LogMsg(servicemanager.EVENTLOG_INFORMATION_TYPE,
                              servicemanager.PYS_SERVICE_STARTED,
                              (self._svc_name_, ''))
        self.main()

    def start(self):
        '''
        Override to add logic before the start
        eg. running condition
        '''
        pass

    def stop(self):
        '''
        Override to add logic before the stop
        eg. invalidating running condition
        '''
        pass

    def main(self):
        '''
        Main class to be ovverridden to add logic
        '''
        pass

def demo():
    class PythonCornerExample(WinService):
        _svc_name_ = "PythonCornerExample"
        _svc_display_name_ = "Python Corner's Winservice Example"
        _svc_description_ = "That's a great winservice! :)"

        def start(self):
            self.isrunning = True

        def stop(self):
            self.isrunning = False

        def main(self):
            import random
            from pathlib import Path
            import time
            while self.isrunning:
                random.seed()
                x = random.randint(1, 1000000)
                Path(f'D:/Projects/PythonProjects/pywinschedule/demo/{x}.txt').touch()
                time.sleep(5)

    PythonCornerExample.parse_command_line()
# entry point of the module: copy and paste into the new module
# ensuring you are calling the "parse_command_line" of the new created class
if __name__ == '__main__':
    demo()